# -*- coding: utf-8 -*-
import requests


class Sms:
	def __init__(self, api_key):
		self.key = api_key
		self.url = 'http://sms-activate.ru/stubs/handler_api.php'

	def request(self, action):
		try:
			print(action.data)
			params = {**{'api_key': self.key}, **action.data}
			response = requests.get(self.url, params)
			return response.text
		except (ConnectionError, TimeoutError):
			return 'NO_CONNECTION'



# -*- coding: utf-8 -*-


class SmsTypes:
	class Country:
		RU = '0'  # Россия (Russia)
		UA = '1'  # Украина (Ukraine)
		KZ = '2'  # Казахстан (Kazakhstan)
		CN = '3'  # Китай (China)
		PH = '4'  # Филиппины (Philippines)
		MM = '5'  # Мьянма (Myanmar)
		ID = '6'  # Индонезия (Indonesia)
		MY = '7'  # Малайзия (Malaysia)
		# KE = '8'  # Кения (Kenya) – temp disable
		# TZ = '9'  # Танзания (Tanzania) – temp disable
		VN = '10'  # Вьетнам (Vietnam)
		KG = '11'  # Кыргызстан (Kyrgyzstan)
		US = '12'  # США (USA)
		IL = '13'  # Израиль (Israel)
		HK = '14'  # Гонконг (Hong Kong)
		PL = '15'  # Польша (Poland)
		UK = '16'  # Великобритания/Англия (United Kingdom)
		MG = '17'  # Мадагаскар (Madagascar)
		CG = '18'  # Конго (Congo)
		NG = '19'  # Нигерия (Nigeria)
		MO = '20'  # Макао (Macau)
		EG = '21'  # Египет (Egypt)
		IE = '23'  # Ирландия (Ireland)
		KH = '24'  # Камбоджа (Cambodia)
		LA = '25'  # Лаос (Lao)
		HT = '26'  # Гаити (Haiti)
		CI = '27'  # Кот д'Ивуар (République de Côte d'Ivoire)
		GM = '28'  # Гамбия (Gambia)
		RS = '29'  # Сербия (Serbian)
		YE = '30'  # Йемен (Yemen)
		ZA = '31'  # ЮАР (South Africa)
		RO = '32'  # Румыния (Romania)
		EE = '34'  # Эстония (Estonia)
		AZ = '35'  # Азербайджан (Azerbaijan)
		CA = '36'  # Канада (Canada)
		MA = '37'  # Марокко (Morocco)
		GH = '38'  # Гана (Ghana)
		AR = '39'  # Аргентина (Argentina)
		UZ = '40'  # Узбекистан (Uzbekistan)
		CM = '41'  # Камерун (Cameroon)
		TG = '42'  # Чад (Chad)
		DE = '43'  # Германия (Germany)
		LT = '44'  # Литва (Lithuania)
		HR = '45'  # Хорватия ( Croatia)
		IQ = '47'  # Ирак (Iraq)
		NL = '48'  # Нидерланды (Netherlands)


	class Status:
		Cancel = '-1'
		SmsSent = '1'
		OneMoreCode = '3'
		End = '6'
		AlreadyUsed = '8'

	class Operator:
		MTS = 'mts'
		Beeline = 'beeline'
		Megafon = 'megafon'
		TELE2 = 'tele2'
		any = 'any'



# -*- coding: utf-8 -*-
from .models import ServiceModel
from .helper import object_factory


class ServiceStorage:
	names = {
		'VkCom': 'vk_0', 'Odnoklassniki': 'ok_0', 'Whatsapp': 'wa_0', 'Viber': 'vi_0', 'Telegram': 'tg_0',
		'WeChat': 'wb_0', 'Google': 'go_0', 'YouTube': 'go_0', 'Gmail': 'go_0', 'Avito': 'av_0',
		'AvitoSmsForwarding': 'av_1', 'Facebook': 'fb_0', 'Twitter': 'tw_0', 'AnyOther': 'ot_0',
		'AnyOtherSmsForwarding': 'ot_1',
		'Uber': 'ub_0', 'Qiwi': 'qw_0', 'GettTaxi': 'gt_0', 'OlxUA': 'sn_0', 'Instagram': 'ig_0',
		'SeoSprint': 'ss_0', 'Youla': 'ym_0', 'YoulaSmsForwarding': 'ym_1', 'MailRu': 'ma_0', 'Microsoft': 'mm_0',
		'Messenger': 'uk_0', 'LineMessenger': 'me_0', 'Yahoo': 'mb_0', 'DrugVokrug': 'we_0', 'FiveOrochka': 'bd_0',
		'TencentQQ': 'kp_0', 'WOG': 'dt_0', 'Yandex': 'ya_0', 'YandexSmsForwarding': 'ya_1', 'Steam': 'mt_0',
		'Tinder': 'oi_0', 'Mamba': 'fd_0', 'DromRu': 'zz_0', 'KakaoTalk': 'kt_0', 'AOL': 'pm_0', 'LinkedIN': 'tn_0'}


class SmsService:
	def __init__(self):
		for name, short_name in ServiceStorage.names.items():
			object = object_factory(
				name,
				base_class=ServiceModel,
				argnames=['__service_short_name', '__count_slot']
			)(__service_short_name=short_name, __count_slot=0)
			setattr(self, '_' + name, object)

	@property
	def VkCom(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._VkCom

	@property
	def Odnoklassniki(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Odnoklassniki

	@property
	def Whatsapp(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Whatsapp

	@property
	def Viber(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Viber

	@property
	def Telegram(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Telegram

	@property
	def WeChat(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._WeChat

	@property
	def Google(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Google

	@property
	def YouTube(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._YouTube

	@property
	def Gmail(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Gmail

	@property
	def Avito(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Avito

	@property
	def AvitoSmsForwarding(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._AvitoSmsForwarding

	@property
	def Facebook(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Facebook

	@property
	def Twitter(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Twitter

	@property
	def AnyOther(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._AnyOther

	@property
	def AnyOtherSmsForwarding(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._AnyOtherSmsForwarding

	@property
	def Uber(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Uber

	@property
	def Qiwi(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Qiwi

	@property
	def GettTaxi(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._GettTaxi

	@property
	def OlxUA(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._OlxUA

	@property
	def Instagram(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Instagram

	@property
	def SeoSprint(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._SeoSprint

	@property
	def Youla(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Youla

	@property
	def YoulaSmsForwarding(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._YoulaSmsForwarding

	@property
	def MailRu(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._MailRu

	@property
	def Microsoft(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Microsoft

	@property
	def Messenger(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Messenger

	@property
	def LineMessenger(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._LineMessenger

	@property
	def Yahoo(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Yahoo

	@property
	def DrugVokrug(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._DrugVokrug

	@property
	def FiveOrochka(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._FiveOrochka

	@property
	def TencentQQ(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._TencentQQ

	@property
	def WOG(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._WOG

	@property
	def Yandex(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Yandex

	@property
	def YandexSmsForwarding(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._YandexSmsForwarding

	@property
	def Steam(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Steam

	@property
	def Tinder(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Tinder

	@property
	def Mamba(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._Mamba

	@property
	def DromRu(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._DromRu

	@property
	def KakaoTalk(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._KakaoTalk

	@property
	def AOL(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._AOL

	@property
	def LinkedIN(self):
		"""
		:rtype: smsactivateru.models.ServiceModel
		"""
		return self._LinkedIN



# -*- coding: utf-8 -*-
from .helper import object_factory
from .models import ErrorsModel


class ErrorStorage:
	codes = {
		'NO_NUMBERS':    {
			'text':  'Нет свободных номеров для приёма смс от текущего сервиса',
			'class': 'NoFreeNumber'
		},
		'NO_BALANCE':    {
			'text':  'Закончился баланс',
			'class': 'NoBalance'
		},
		'BAD_ACTION':    {
			'text':  'Некорректное действие (параметр action)',
			'class': 'InvalidAction'
		},
		'BAD_SERVICE':   {
			'text':  'Некорректное наименование сервиса (параметр service)',
			'class': 'InvalidService'
		},
		'BAD_KEY':       {
			'text':  'Неверный API ключ доступа',
			'class': 'InvalidApiKey'
		},
		'ERROR_SQL':     {
			'text':  'Один из параметров имеет недопустимое значение',
			'class': 'InvalidData'
		},
		'NO_ACTIVATION': {
			'text':  'Указанного id активации не существует',
			'class': 'InvalidId'
		},
		'BAD_STATUS':    {
			'text':  'Попытка установить несуществующий статус',
			'class': 'InvalidStatus'
		},
		'STATUS_CANCEL': {
			'text':  'Текущая активация отменена и больше не доступна',
			'class': 'CanceledActivation'
		},
		'BANNED':        {
			'text':  'Аккаунт заблокирован',
			'class': 'BannedUser'
		},
		'NO_CONNECTION': {
			'text':  'Нет соединения с серверами sms-activate',
			'class': 'ConnectionError'
		}
	}


def error_handler(function: classmethod):
	def wrapper(self, response, *args, **kwargs):
		for key in ErrorStorage.codes.keys():
			if key in response:
				except_class = object_factory(
					ErrorStorage.codes[key]['class'],
					base_class=ErrorsModel,
					argnames=['message', 'code'])
				raise except_class(message=ErrorStorage.codes[key]['text'], code=key)
		return function(self, response, *args, **kwargs)
	return wrapper



# -*- coding: utf-8 -*-
import time

import smsactivateru


class SmsActivation:
	"""
	This is simple worker!
	For hand-settings any other params – check /example/custom.py
	"""

	def __init__(self, activation_id, number, wrapper):
		self.__id = activation_id
		self.__number = number
		self.__wrapper = wrapper
		self.__last_code = str()

	def cancel(self):
		set_status = smsactivateru.SetStatus(
			id=self.__id,
			status=smsactivateru.SmsTypes.Status.Cancel
		)
		if self.wrapper:
			return set_status.request(self.wrapper)
		return set_status

	def mark_as_used(self):
		set_status = smsactivateru.SetStatus(
			id=self.__id,
			status=smsactivateru.SmsTypes.Status.AlreadyUsed
		)
		if self.wrapper:
			return set_status.request(self.wrapper)
		return set_status

	def was_sent(self):
		set_status = smsactivateru.SetStatus(
			id=self.__id,
			status=smsactivateru.SmsTypes.Status.SmsSent
		)
		if self.wrapper:
			return set_status.request(self.wrapper)
		return set_status

	def wait_code(self, timeout=1200, callback=None, not_end=False, *args, **kwargs):
		"""
		:param wrapper: obj for work with sms-activate
		:param timeout: timeout waiting of code from sms in secs. 1200 - 20 min, this is max time of a live session.
		:param callback: function for eval before getting code
		:param not_end:
		:return: str
		"""
		counter = 0
		while True:
			time.sleep(1)
			counter += 1
			if counter >= timeout:
				raise ('Timeout error')
			response = smsactivateru.GetStatus(id=self.id).request(self.wrapper)
			if response['code'] and not not_end and response['code'] != self.last_code:
				self.__last_code = response['code']
				smsactivateru.SetStatus(
					id=self.id,
					status=smsactivateru.SmsTypes.Status.End
				).request(self.wrapper)
				break
			elif response['code'] and not_end and response['code'] != self.last_code:
				self.__last_code = response['code']
				smsactivateru.SetStatus(
					id=self.id,
					status=smsactivateru.SmsTypes.Status.OneMoreCode
				)
				break
		if callback:
			callback(self.last_code)
		else:
			return self.last_code

	@property
	def id(self):
		return self.__id

	@property
	def phone_number(self):
		return self.__number

	@property
	def wrapper(self):
		return self.__wrapper

	@property
	def last_code(self):
		return self.__last_code



# -*- coding: utf-8 -*-
import inspect


class ErrorsModel(Exception):
	def __init__(self):
		super().__init__(self.message, self.code)


class ServiceModel:
	@property
	def short_name(self):
		return self.__service_short_name

	@property
	def count(self):
		return self.__count_slot

	@count.setter
	def count(self, value):
		self.__count_slot = int(value)


class ActionsModel:
	__action_name = ''
	__response_data = ''

	def __init__(self, current):
		self.__request_data = {'action': self._name}
		self.data = self._build(current)

	def _build(self, frame):
		args, _, _, values = inspect.getargvalues(frame)
		exclude = ['self', 'callback', 'wrapper']
		result = {}
		for i in args:
			if i == 'ref':
				values[i] = 'python' + __name__.split('.')[0][:-2]
			elif not values[i]:
				continue
			if i in exclude:
				continue
			result[i] = values[i]
		return result

	@property
	def data(self):
		return self.__request_data

	@data.setter
	def data(self, value):
		self.__request_data = {**self.__request_data, **value}

	@property
	def _name(self):
		return self.__action_name

	@_name.setter
	def _name(self, value):
		self.__action_name = value



# -*- coding: utf-8 -*-
import inspect
import sys


def all_classes_name(module: str):
	clsmembers = inspect.getmembers(sys.modules[module], inspect.isclass)
	result = []
	for class_tuple in clsmembers:
		result.append(class_tuple[0])
	return result


def object_factory(name, base_class, argnames):
	def __init__(self, **kwargs):
		for key, value in kwargs.items():
			if key not in argnames:
				raise TypeError('Argument {} not valid for {}'.format(key, self.__class__.__name__))
			setattr(self, key, value)
		base_class.__init__(self)

	newclass = type(name, (base_class,), {'__init__': __init__})
	return newclass


def singleton(class_):
	instances = {}

	def get_instance(*args, **kwargs):
		if class_ not in instances:
			instances[class_] = class_(*args, **kwargs)
		return instances[class_]

	return get_instance



# -*- coding: utf-8 -*-
from .sms import Sms
from .types import SmsTypes
from .actions import GetBalance, GetFreeSlots, GetNumber, SetStatus, GetStatus
from .services import SmsService
from .activations import SmsActivation

__author__ = 'tezmen'
__version__ = '1.0'
__contact__ = 'https://t.me/tezmen'


# -*- coding: utf-8 -*-
from os import path
from setuptools import setup

"""
@author: tezmen
@contact: https://t.me/tezmen
@license Apache License, Version 2.0, see LICENSE file
Copyright (C) 2018
"""


def long_description():
	"""Build the description from README file """
	this_dir = path.abspath(path.dirname(__file__))
	with open(path.join(this_dir, 'README.md'), encoding='utf-8') as f:
		return f.read()


def requirements():
	"""Build the requirements list for this project"""
	requirements_list = list()
	with open('requirements.txt') as pc_requirements:
		for install in pc_requirements:
			requirements_list.append(install.strip())
	return requirements_list


setup(
	name='smsactivateru',
	version='1.2.3',
	long_description=long_description(),
	long_description_content_type='text/markdown',
	description='Wrapper for automatic reception of SMS-messages by sms-activate.ru',
	author='tezmen',
	license='Apache License, Version 2.0, see LICENSE file',
	keywords='sms, revice, sms-activate-ru, autoreg',
	author_email='tezmenpro@gmail.com',
	url='https://github.com/tezmen/python-sms-activate-ru',
	download_url='https://github.com/tezmen/python-sms-activate-ru/archive/master.zip',
	packages=['smsactivateru'],
	install_requires=requirements(),
	classifiers=[
		'License :: OSI Approved :: Apache Software License',
		'Operating System :: OS Independent',
		'Environment :: Console',
		'Development Status :: 3 - Alpha',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6',
		'Programming Language :: Python :: 3.7',
		'Programming Language :: Python :: Implementation :: PyPy',
	]
)



# -*- coding: utf-8 -*-
import inspect
import json

from .activations import SmsActivation
from .errors import error_handler
from .models import ActionsModel
from .services import SmsService, ServiceStorage


class GetBalance(ActionsModel):
	_name = 'getBalance'

	def __init__(self):
		super().__init__(inspect.currentframe())

	@error_handler
	def __response_processing(self, response):
		return float(response.split(':')[1])

	def request(self, wrapper):
		"""
		:rtype: int
		"""
		response = wrapper.request(self)
		return self.__response_processing(response)


class GetFreeSlots(ActionsModel):
	_name = 'getNumbersStatus'

	def __init__(self, country=None, operator=None):
		super().__init__(inspect.currentframe())

	@error_handler
	def __response_processing(self, response):
		service_list = json.loads(response)

		service_obj = SmsService()
		for name, short_name in ServiceStorage.names.items():
			getattr(service_obj, name).count = int(service_list[short_name])
		return service_obj

	def request(self, wrapper):
		"""
		:rtype: smsactivateru.services.SmsService
		"""
		response = wrapper.request(self)
		return self.__response_processing(response)


class GetNumber(ActionsModel):
	_name = 'getNumber'

	def __init__(self, service, country=None, operator=None, forward=False):
		service = getattr(service, '__service_short_name').split('_')[0]
		forward = int(forward)
		super().__init__(inspect.currentframe())

	@error_handler
	def __response_processing(self, response, wrapper):
		data = response.split(':')
		return SmsActivation(data[1], data[2], wrapper)

	def request(self, wrapper):
		"""
		:rtype: smsactivateru.activations.SmsActivation
		"""
		response = wrapper.request(self)
		return self.__response_processing(response, wrapper=wrapper)


class GetStatus(ActionsModel):
	_name = 'getStatus'

	def __init__(self, id):
		super().__init__(inspect.currentframe())

	@error_handler
	def __response_processing(self, response):
		data = {'status': response, 'code': None}
		if ':' in response:
			data['status'] = response.split(':', 1)[0]
			data['code'] = response.split(':', 1)[1]
		return data

	def request(self, wrapper):
		"""
		:rtype: dict
		"""
		response = wrapper.request(self)
		return self.__response_processing(response)


class SetStatus(ActionsModel):
	_name = 'setStatus'

	def __init__(self, id, status, forward=False):
		forward = int(forward)
		super().__init__(inspect.currentframe())

	@error_handler
	def __response_processing(self, response):
		data = {'status': response}
		return data

	def request(self, wrapper):
		"""
		:rtype: dict
		"""
		response = wrapper.request(self)
		return self.__response_processing(response)
